import { Component, Input, OnInit, ViewChild } from '@angular/core';
import { MatButton } from '@angular/material/button';
import { MatTableDataSource } from '@angular/material/table';
import { AccountEntryService, AccountService } from '../data-object-service';
import { Account, AccountEntry, NULL_AccountEntry } from '../data-objects';
import { ExtApiService } from '../ext-data-object-service';
import { Saldo } from '../ext-data-objects';
import { MessageService } from '../message.service';

@Component({
  selector: 'app-account',
  templateUrl: './account.component.html',
  styleUrls: ['./account.component.css']
})
export class AccountComponent implements OnInit {

  @Input() selectedAccountId: number
  @ViewChild('addAccountEntryButton') addAccountEntryButton: MatButton

  collapse: boolean = false

  account: Account
  accountEntries: AccountEntry[]
  accountEntriesDataSource: MatTableDataSource<AccountEntry>
  accountEntriesDisplayedColumns: string[] = [ "description", "amount", "createdAt" ]
  saldo: Saldo


  newAccountEntry: AccountEntry = NULL_AccountEntry


  constructor(
    private accountService: AccountService,
    private accountEntryService: AccountEntryService,
    private extApiService: ExtApiService,
    private messageService: MessageService
  ) { }

  async getAccount(): Promise<void> {
    try {
      if (this.selectedAccountId) {
        this.messageService.add(`Trying to load account ${this.selectedAccountId} and entries`)
        this.account = await this.accountService.getAccount(this.selectedAccountId)
        this.messageService.add(`Account: ${JSON.stringify(this.account, undefined, 4)}`)
        this.getAccountEntries()
      }
    } catch (err) {
      this.messageService.add(`Error in getAccount: ${JSON.stringify(err, undefined, 4)}`)
    }
  }

  async getAccountEntries(): Promise<void> {
    try {
      this.accountEntries = await this.accountEntryService.getAccountEntrysByAccount(this.selectedAccountId)
      this.accountEntries.reverse()
      this.messageService.add(`AccountEntries: ${JSON.stringify(this.accountEntries, undefined, 4)}`)
      this.accountEntriesDataSource = new MatTableDataSource<AccountEntry>(this.accountEntries)
      this.saldo = await this.extApiService.getAccountSaldo(this.selectedAccountId)
    } catch (err) {
      this.messageService.add(`Error in getAccountEntries: ${JSON.stringify(err, undefined, 4)}`)
    }
  }

  async addAccountEntry(): Promise<void> {
      try {
        this.addAccountEntryButton.disabled = true
        this.newAccountEntry.account = this.account.id
        this.messageService.add(`addAccountEntry: ${ JSON.stringify(this.newAccountEntry, undefined, 4) }`)
        this.newAccountEntry = await this.accountEntryService.postAccountEntry(this.newAccountEntry)
        this.messageService.add(`New accountEntry created: ${this.newAccountEntry.id}`)
        this.newAccountEntry = { 'account': this.account.id, 'amount': undefined, 'created_at': '', 'description': '', 'id': 0 }
        this.getAccountEntries()
      } catch (err) {
        this.messageService.add(`Error in addAccountEntry: ${JSON.stringify(err, undefined, 4)}`)
        } finally {
        this.addAccountEntryButton.disabled = false
      }
  }

  ngOnInit(): void {
    this.messageService.add(`AccountComponent.ngOnInit, account: ${this.selectedAccountId}`)
    this.getAccount()
  }

}
