import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { MessageService } from '../message.service';
import { TokenService } from '../token.service';


@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css']
})
export class LoginComponent implements OnInit {
  form: FormGroup;
  public loginInvalid = false;
  private formSubmitAttempt = false;
  private returnUrl: string;

  constructor(private fb: FormBuilder,
    private route: ActivatedRoute,
    private router: Router,
    private tokenService: TokenService,
    private messageService: MessageService) {
      this.returnUrl = this.route.snapshot.queryParams.returnUrl || '/'
      this.messageService.add(`Return URL is ${this.returnUrl}`)
      this.form = this.fb.group({
        username: ['', Validators.required],
        password: ['', Validators.required]
      })
  }

  ngOnInit(): void {
    if (this.tokenService.checkAuthenticated()) {
      this.router.navigate([this.returnUrl])
    }
  }

  async onSubmit(): Promise<void> {
    this.loginInvalid = false;
    this.formSubmitAttempt = false;
    if (this.form.valid) {
      try {
        const username = this.form.get('username')?.value;
        const password = this.form.get('password')?.value;
        await this.tokenService.login(username, password);
        this.router.navigate([this.returnUrl])
      } catch (err) {
        this.messageService.add(`Login err: ${err.message}`)
        this.loginInvalid = true;
      }
    } else {
      this.formSubmitAttempt = true;
    }
  }

}
